use mongodb::{Client, options::{ClientOptions, Credential}};
use std::time::{self, SystemTime};

mod packet;
use packet::Packet;

#[tokio::main]
async fn main() {
    // Parse a connection string into an options struct.
    let mut client_options = ClientOptions::parse("mongodb://localhost:27017").await.unwrap();

    // Manually set an option.
    client_options.app_name = Some("My App".to_string());

    let credential_builder = Credential::builder()
                            .username(String::from(env!("mongo_username")))
                            .password(String::from(env!("mongo_password")));

    client_options.credential = Some(credential_builder.build());
    
    // Get a handle to the deployment.
    let client = Client::with_options(client_options).unwrap();
    let db = client.database("test");
    match db.create_collection("test", None).await {
        Err(_) => {
            db.collection::<Packet>("test").drop(None).await.unwrap();
        },
        _ => {}
    }
    
    const NUM_QUERIES: u32 = 5_000;
    let collection = db.collection::<Packet>("test");
    let mut queries: Vec<std::time::Duration> = vec![];
    let start_time = SystemTime::now();

    for _ in 0..NUM_QUERIES {
        let query_time = SystemTime::now();
        
        collection.insert_one(Packet{ 
            time: SystemTime::now(), 
            text: format!("Current Unix time: {:?}", SystemTime::now().duration_since(time::UNIX_EPOCH).unwrap())
        }, None).await.unwrap();

        let duration = SystemTime::now().duration_since(query_time).unwrap();
        queries.push(duration);
    }
    
    for (i, duration) in queries.iter().enumerate() {
        println!("{}: {}.{} milliseconds", i, duration.as_millis(), duration.as_micros());
    }

    println!("Ran {} queries in {} seconds.", NUM_QUERIES, SystemTime::now().duration_since(start_time).unwrap().as_secs_f32());
}